CPU = 16F877A
MHZ = 10
CONFIG 16250

;Simplified code for "Creepy Hybrid"
;based on Lynxmotion AH3-R code
;
;05/20/09 ALM
; creepy2  05/27/09 to HBRobotics meeting (walking)
;
;09/27/09 creepy2Mb Ported to MBasic for QuikFlash board, functional
;
; http://www.picbook.com/bookinfo/QwikFlash.pdf   QuikFlash schematic
:
; Use Basic Micro ATOM IDE 05.3.1.3
; Use Basic Micro MBasic Pro 5.2.1.1  16F877A 10 MHz
; SSC-32  jumpered for 38.4K
; COM -OR- PS2 wireless joystick controller
;
; 3DOF-A legs
; 
;
; Build 1 right leg and 1 left leg 3DOF-A
; Position the SSC-32 card with the PC serial port looking forward
;
; right leg connections :
; Front Right leg : Hip Horizontal : pin 8, Hip Vertical : pin 9, Knee : pin 10
;
; left leg connections :
; Front Left leg : Hip Horizontal : pin 24, Hip Vertical : pin 25, Knee : pin 26
;
; Normal Operating mode :
; Move the robot (2 axis translation) with the right joystick
; Turn with the left joystick X axis
; move body Up/Down with the left joystick Y axis
;
; Go to standby position with the "Triangle" button, press again to go to walk position
; All = 1500 posture with the "Circle" button 
;
; Change the legs vertical position with the digital Up/Down buttons
; Change the gait speed with digital right/left buttons
;
; "R3 button" (right joystick push button) : Horn
;
;--------------------------------------------------------------------
; Constants

; I/O pin defines for 16F877A or 18F4620 on QuikFlash board
RA0  con 0
RA1  con 1 
RA2  con 2 
RA3  con 3 
RA4  con 4 
RA5  con 5 
RA6  con 6 
RA7  con 7
RB0  con 8 
RB1  con 9
RB2  con 10
RB3  con 11
RB4  con 12
RB5  con 13
RB6  con 14
RB7  con 15
RC0  con 16
RC1  con 17
RC2  con 18
RC3  con 19
RC4  con 20
RC5  con 21
RC6  con 22
RC7  con 23
RD0  con 24
RD1  con 25
RD2  con 26
RD3  con 27
RD4  con 28
RD5  con 29
RD6  con 30 
RD7  con 31
RE0  con 32
RE1  con 33
RE2  con 34

;5K POT
POT    con RA5

; Moved LEDs for QwikFlash 
LED_D2 con RA4	; heartbeat
LED_D4 con RA3	; left
LED_D5 con RA2	; center
LED_D6 con RA1	; right

; temperature sensor
TMP1  con RA0


;SSC-32 board TTL com connection pin 8
SSC32 con RB0

; although we don't have a speaker
SPKR  con RB1

;PS2 Controller / (BotBoard II	pins 12, 13, 14, 15)
DAT   con RB2	; data
CMD   con RB3	; command
SEL   con RB4	; select
CLK   con RB5	; clock

; D/A converter
DAC_CS   con RC0 ; chip select
H3_INPUT con RC1 ; C1/CCP2
H2_INPUT con RC2 ; C2/CCP1
DAC_SCLK con RC3 ; D/A converter clock
UNUSED   con RC4 ;
DAC_DIN  con RC5 ;

; com port w/ RS-232 lever converter
RX       con RC6 ; com port
TX       con RC7 ; com port

; shaft encoder
ENC_A  con RD0
ENC_B  con RD1
;S_OUT con RD2
SW3    con RD3

;SW1 is power
;SW2 is RESET

;2 line X 8 LCD
LCD_D4 con RD4
LCD_D5 con RD5
LCD_D6 con RD6
LCD_D7 con RD7
LCD_RS con RE0
LCD_E  con RE1
LCD_RW con RE2


DeadZone con 28
PadMode con $79

;Leg maximum excursion angles and pulses

HipH_AngleMin con 21		;30
HipH_AngleMax con 107		;150
HipH_PulseMin con 910		
HipH_PulseMax con 2090		

HipV_AngleMin con 25		;35
HipV_AngleMax con 103		;145
HipV_PulseMin con 960	
HipV_PulseMax con 2040	

Knee_AngleMin con 36		;50
Knee_AngleMax con 107		;150
Knee_PulseMin con 1107		
Knee_PulseMax con 2090

; 3DOF-A Leg Dimensions		(TibiaAngle constant = 0)

HipV_HipH    con 38    ;1.50"  = 38mm (1.50 * 25.4)
Femur_Length con 57    ;2.25"  = 57mm (2.25 * 25.4)
Tibia_Length con 124   ;4.875" = 124mm (4.875 * 25.4)
TibiaAngle   con 0     ;range is -20 (interior) to 20 (exterior), 0 is vertical

LegUpShiftMin con 30
LegUpShiftMax con 70

; leg position index values (0 is Right aft, 2 is Right fwd, 3 is Left fwd, 5 is Left aft

RightLeg con 2
LeftLeg  con 3
cr con 13

;ACos table
data	64,64,63,63,63,62,62,62,61,61,61,60,60,60,59,59,|
	59,59,58,58,58,57,57,57,56,56,56,55,55,55,54,54,|
	54,53,53,53,52,52,52,51,51,51,50,50,50,49,49,49,|
	48,48,48,47,47,46,46,46,45,45,45,44,44,44,43,43,|
	42,42,42,41,41,41,40,40,39,39,39,38,38,37,37,37,|
	36,36,35,35,35,34,34,33,33,32,32,31,31,31,30,30,|
	29,29,28,28,27,27,26,25,25,24,24,23,23,22,21,21,|
	20,19,19,18,17,16,15,15,14,13,11,10,09,07,05,00


; defines for SSC32 servo numbers (smaller then byte table)

FRHH con  "0"	;Front Right leg : Hip Horizontal : pin 8
FRHH2 con "8"
FRHV con  "0"	;Front Right leg : Hip Vertical   : pin 9
FRHV2 con "9"
FRK con   "1"	;Front Right leg : Knee	  	  : pin 10
FRK2 con  "0"

FLHH con  "2"	;Front Left leg : Hip Horizontal  : pin 24
FLHH2 con "4"
FLHV con  "2"	;Front Left leg : Hip Vertical 	  : pin 25
FLHV2 con "5"
FLK con   "2"	;Front Left leg : Knee	  	  : pin 26
FLK2 con  "6"

;--------------------------------------------------------------------				
; Variables

Index var Byte
DualShock var Byte(7)
DS2Mode var Byte
LastButton var Byte(2)
Steps var Byte
FlipFlop var Bit
FlagOff var Bit
MovesDelay var Sbyte
HeightAdjust var Bit

Steering var Sbyte
Height var Sbyte
LastHeight var Sbyte
HeightLimit var Byte
XSpeed var Sbyte
YSpeed var Sbyte

Rotate var Sbyte(6)	; room for 6
DCoord var Byte
DAngle var Byte
NbSteps var Byte
GaitSpeed var Byte
GaitSpeedTmp var Byte
StepFlag var Byte

; temporary storage

TmpXPos var Long
TmpYPos var Long
TmpZPos var Long
TmpDistance var Long
TmpC var Word
TmpCC var Long
TmpCos var Long
TmpAngle var Sbyte

; leave the arrays big enough for 6 legs

XPos var Sword(6)
YPos var Sword(6)
ZPos var Sword(6)
XPos2 var Sword(6)
YPos2 var Sword(6)
ZPos2 var Sword(6)

Distance var Sword

; servo angles

HipH_Angle var Sbyte
HipV_Angle var Sword
Knee_Angle var Sword

; servo pulse counts

HipH_Pulse var Word(6)	; room for 6
HipV_Pulse var Word(6)
Knee_Pulse var Word(6)

tmp var byte

; flags

LegUpShift var Byte
Freeze 	   var Bit
Tripod     var Bit
LockLegs   var Bit

; no conditional compiles in Mbasic

CheckSum var Byte

;--------------------------------------------------------------------	
;
;Start of Program

;Initialization

clear	; clear RAM

;sethserial H9600  ; works
sethserial H38400 ; works
;sethserial H115200 ; ??

hserstat 0 ;' clear com input buffer

high CLK   ; PS2 clock line

; LED bits for tests
low  LED_D2	;low=on "heartbeat"

; LED bits for tests (high=on)
low LED_D4 ; left
low LED_D5 ; middle
low LED_D6 ; right 

LastButton(0) = 255
LastButton(1) = 255

HeightLimit = 25 - ABS(TibiaAngle / 2)

HSEROUT["Creepy",cr]	;testing

; Output to 8 x 2 LCD
LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1,[INITLCD2,TWOLINE,CLEAR,HOME,SCR] ; initialize

LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1, ["Creepy  "]	; 1st line

LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1,[SCRRAM+$40]	; move to 2nd line

LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1, ["Hybrid  "]  ; 2nd line


; Initialize SSC-32 with internally stored pulse offsets (previously loaded by Powerpod)
; Run PowerPod and copy two "front leg" data here

serout SSC32,i38400,["#", |
	FRHH,FRHH2,"po42 #",FRHV,FRHV2,"po49 #",FRK,FRK2,"po0 #", |
	FLHH,FLHH2,"po39 #",FLHV,FLHV2,"po71 #",FLK,FLK2,"po0",13]

; set the initial leg positions
gosub H3Init	;set initial leg positions
gosub All1500	; all legs to 1500 (neutral)

; sound pin,[dur mS\freq Hz]	; For MBasic ! 
Sound SPKR,[1000\4235, 2000\4435] ;duration mS  1 sec, 2 sec 

;--------------------------------------------------------------------
; Main program loop
main

; use either serial or PS2 interface (un-comment one only)

gosub SerialQuery	; code for serial commands

; gosub PS2_Query	; alternate code for PS2	


;------------
; Read the PS2 Joystick and a few buttons
; use LED_D5 to indicate command decoded

; Toggle Sleep mode

	if (DualShock(2).bit4 = 0) and LastButton(1).bit4 then 	;"Triangle" Button   sleep mode
		FlagOff = FlagOff ^ 1	; toggle sleep mode on/off
		high LED_D5
	
		if FlagOff then
			gosub All1500
			Gosub InitPos
			pause 144
			
			for Index = 8 to 10	; right leg
				serout SSC32,i38400,["#",DEC Index,"P0"] ; put servos to sleep
			next

			for Index = 24 to 26	; left leg
				serout SSC32,i38400,["#",DEC Index,"P0"] ; put servos to sleep
			next
			
			serout SSC32,i38400,[13]
		else
			gosub H3Init;  waking up! position servos
			gosub All1500; all legs to 1500 (neutral)
		endif
		low LED_D5
	endif
	
	if FlagOff	then
		pause 144
		goto OffEnd
	endif
	
	
; Legs to neutral

	if (MovesDelay <= 0) then
		if (DualShock(2).bit5 = 0) and LastButton(1).bit5 then	;"Circle" Button legs to neutral
			high LED_D5
			gosub All1500
			low LED_D5
		endif
	endif

; Sound the horn!

	if (DualShock(1).bit2 = 0) and LastButton(0).bit2 then	;R3 Button  (Horn)
		high LED_D5
		Sound SPKR,[800\392, 600\370, 200\0, 800\330, 600\294, 200\0, |
			300\330, 100\0, 300\330]
		low LED_D5
	endif

; Get the leg shift up/down requests, within limits!
	
	if DualShock(1).bit4 = 0 then	 	;Up Button test
		LegUpShift = (LegUpShift + 1) max LegUpShiftMax	
		high LED_D5
	elseif DualShock(1).bit6 = 0		;Down Button test
		LegUpShift = (LegUpShift - 1) min LegUpShiftMin	
		high LED_D5
	else
		goto NoSound
	endif

	Sound SPKR,[200\(LegUpShift * 10 + 400)]	; duration ms \ tone

	MovesDelay = 8 + NbSteps  - Steps ; insure legs get back to "Steps = 4" (on ground)
NoSound
	low LED_D5

; Right/Left buttons set gait speed

	if (DualShock(1).bit5 = 0) and LastButton(0).bit5 then 	;"Right" Button test
		GaitSpeedTmp = (GaitSpeedTmp + 1) max 20
;hserout ["+GaitSpeedTemp",dec GaitSpeedTmp,cr]  
		high LED_D5
	elseif (DualShock(1).bit7 = 0) and LastButton(0).bit7	;"Left" Button test
		GaitSpeedTmp = (GaitSpeedTmp - 1) min 3
;hserout ["-GaitSpeedTemp",dec GaitSpeedTmp,cr]  
		high LED_D5
	else
		goto NoSound1
	endif
	
	Sound SPKR,[200\(50-GaitSpeedTmp) * 35 + 400]

	MovesDelay = 8 + NbSteps  - Steps ; insure legs get back to "Steps = 4" (on ground)
NoSound1
	low LED_D5

	GaitSpeed = GaitSpeedTmp min 3


; get left and right joystick x and y values, observe dead zone

	for Index = 3 to 6	
		if DualShock(Index) > (128 + DeadZone) 	then
			DualShock(Index) = DualShock(Index) - DeadZone/2
		elseif DualShock(Index) < (128 - DeadZone)
			DualShock(Index) = DualShock(Index) + DeadZone/2
		else
			DualShock(Index) = 128
		endif
	next

	YSpeed = DualShock(3) - 128			; Right Stick Horizontal
	XSpeed = DualShock(4) - 128			; Right Stick Vertical
	
	Steering = DualShock(5) - 128		; Left Stick Horizontal
	StepFlag = (GaitSpeedTmp - 2) max 3

	; adjust height incrementally	
	Height = LastHeight + (((DualShock(6) - 128) / 5) - LastHeight) / StepFlag; Left Stick Vertical
	Height = (Height Max HeightLimit) min - HeightLimit


; set MovesDelay flag if a command received.

	if Steering or (Height <> LastHeight) or YSpeed or XSpeed then
	MovesDelay = 8 + NbSteps  - Steps  ; insure legs get back to "Steps = 4" (on ground)
	endif

;-----------------------------------------------------------------------
; TIME TO MOVE LEGS!

	if MovesDelay > 0 	then		
		LastHeight = Height
		DCoord = SQR(XSpeed * XSpeed + YSpeed * YSpeed)	; magnitude of move to make
		TmpCos = XSpeed * 127 / DCoord
		
		gosub ACos
		DAngle = TmpAngle	; angle of move to make
		
		if YSpeed > 0 	then
			DAngle = 256 - DAngle	; correct it for sign
		endif

		DCoord = DCoord max (128 - ABS(Height) - ABS(TibiaAngle * 2)) ; limit move max


; polar to rectangular conversion for leg moves
; OUR legs are facing forward, = 180 degrees (127)

		for Index = RightLeg to LeftLeg	
			XPos2(Index) = -(DCoord * COS(DAngle + 127) / 300) ; 127 =  180 degrees
			ZPos2(Index) = -(DCoord * SIN(DAngle + 127) / 300) ; 127 =  180 degrees
		next


; count through the steps needed in the gait cycle, flip legs after each gait cycle
		
		Steps = Steps + 1
		
		if Steps > NbSteps 	then
			Steps = 1
			FlipFlop = FlipFlop ^ 1	;toggle up/down leg for next gait cycle
			NbSteps = GaitSpeed
			StepFlag = NbSteps - Steps + 1 
		endif

; Two legs, alternate between two leg positons

		Tripod = FlipFlop	; one leg up
		gosub AorB_Down		; calculate new positon for the down leg

		Freeze = Steps < NbSteps ; freeze off for last step of gait ?
		
		Tripod = FlipFlop ^ 1	; now other leg up
		gosub AorB_Up		; calculate new positon for the up leg


; Calculate angles for leg joints (servos) of the two legs

		for Index = RightLeg to LeftLeg	

			;Distance and HipH_Angle with XZ
			
; pythagorean theorm  calc hip Horizontal angle

			Distance = SQR((XPos(Index) * XPos(Index) + ZPos(Index) * ZPos(Index)) / 4) * 2
			TmpCos = ZPos(Index) * 127 / Distance

			Distance = Distance + TibiaAngle ; Tibia angle for special legs (3DOF-C)

			Gosub ACos
			
			HipH_Angle = (TmpAngle max HipH_AngleMax) min HipH_AngleMin
			
			;----------

			;Set Angle
			TmpXPos = XPos(Index) * 127
			TmpYPos = (YPos(Index) * 127) max 0
			TmpZPos = ZPos(Index) * 127
			TmpDistance = ((Distance - HipV_HipH) * 127) min 0
			
			TmpCC = (TmpYPos * TmpYPos + TmpDistance * TmpDistance) / 16129
			TmpC = SQR(TmpCC)

; Cosine law calc hip Vertical angle (1st part)

			TmpCos = (Femur_Length * Femur_Length + Tibia_Length * Tibia_Length - TmpCC)
			TmpCos = TmpCos * 127 / (2 * Femur_Length * Tibia_Length)
			
			gosub ACos
			
			Knee_Angle = (TmpAngle max Knee_AngleMax) min Knee_AngleMin

			TmpCos = -TmpYPos / TmpC
			
			gosub ACos
			
			HipV_Angle = TmpAngle

; Cosine law calc hip Vertical angle (2nd part)

			TmpCos = (Femur_Length * Femur_Length - Tibia_Length * Tibia_Length + TmpCC) 
			TmpCos = TmpCos * 127 / (2 * Femur_Length * TmpC) 
			
	    	gosub ACos
		    
	    	HipV_Angle = ((TmpAngle + HipV_Angle) max HipV_AngleMax) min HipV_AngleMin
		    
		    	;----------      

			HipH_Angle = HipH_Angle + Rotate(Index)
			
			HipH_Pulse(Index) =  (((HipH_PulseMax - HipH_PulseMin) * (HipH_Angle - HipH_AngleMin) |
				/ (HipH_AngleMax - HipH_AngleMin) + HipH_PulseMin) max HipH_PulseMax) min HipH_PulseMin

			HipV_Pulse(Index) = (((HipV_PulseMax - HipV_PulseMin) * (HipV_Angle - HipV_AngleMin) |
				/ (HipV_AngleMax - HipV_AngleMin) + HipV_PulseMin) max HipV_PulseMax) min HipV_PulseMin

			Knee_Pulse(Index) = (((Knee_PulseMax - Knee_PulseMin) * (Knee_Angle - Knee_AngleMin) |
				/ (Knee_AngleMax - Knee_AngleMin) + Knee_PulseMin) max Knee_PulseMax) min Knee_PulseMin
		next
		

; output servo positions of our two legs to SSC32

		serout SSC32,i38400, [ |
			"#",FRHH,FRHH2,"P",DEC HipH_Pulse(2),       "#",FRHV,FRHV2,"P",DEC HipV_Pulse(2), |
			"#",FRK,FRK2,"P",DEC Knee_Pulse(2),         "#",FLHH,FLHH2,"P",DEC HipH_Pulse(3), |
			"#",FLHV,FLHV2,"P",DEC 3000 - HipV_Pulse(3),"#",FLK,FLK2,"P",DEC 3000 - Knee_Pulse(3), |
			"T180",13]

	else	; not finished moving yet
		pause 144
		
		if MovesDelay < -17 then
			MovesDelay = NbSteps  - Steps + 1
;hserout ["MovesDelay",dec MovesDelay,cr]  
		EndIf	
	endif
	
	MovesDelay = MovesDelay - 1
OffEnd

; keep track of button pushes so that we do not do them multiple times

	LastButton(0) = DualShock(1)
	LastButton(1) = DualShock(2)

goto main  ; the MAIN PROGRAM LOOP

;--------------------------------------------------------------------
; Initialization subroutine

H3Init
	NbSteps = 4	; initially 4 steps in gait
	GaitSpeedTmp = NbSteps
	LegUpShift = 35

	for Index = RightLeg to LeftLeg	; initial leg positions
		XPos(Index) = HipV_HipH + Femur_Length
		YPos(Index) = - Tibia_Length
		ZPos(Index) = 0
	next

	Gosub InitPos	; move to initial leg positions
	
	MovesDelay = 8
	
return

;--------------------------------------------------------------------
; All legs to 1500 (neutral) subroutine 
All1500
	height = 0
	LastHeight = 0

	for Index = 8 to 10	; right leg to neutral position	
		serout SSC32,i38400,["#",DEC Index,"P1500"]
	next

	for Index = 24 to 26	; left leg to neutral position	
		serout SSC32,i38400,["#",DEC Index,"P1500"]
	next

	serout SSC32,i38400,["T576",13]

	pause 576

	Steps = NbSteps
	
return
	
;--------------------------------------------------------------------
; initial servo position subroutine
; gracefully get the robot up

InitPos
	serout SSC32,i38400,["#", |		; move the V hips and then the knees
		FRHV,FRHV2,"P2100#",FRK,FRK2,"P1800#", |
		FLHV,FLHV2,"P900#" ,FLK,FLK2,"P1200 T1152",13]

	pause 576
	
	serout SSC32,i38400,["#", |		; move the H hips
		FRHH,FRHH2,"P1500#", FLHH,FLHH2,"P1500 T576",13]
		
	pause 576	

return

;--------------------------------------------------------------------
; subroutine calcualte an ArcCos

ACos
	TmpCos = (TmpCos max 127) min -127
	
	if TmpCos < 0 	then
		read -TmpCos,TmpAngle
		TmpAngle = 127 - TmpAngle
	else
		read TmpCos,TmpAngle
	endif
	
return

;--------------------------------------------------------------------
; Down Tripod subroutine
;Tripod 'A' (tripod = 0) or 'B' (Tripod = 1) Down

AorB_Down
	high LED_D6 ; flash led to show activity

;	for Index = 0 + Tripod to 4 + Tripod Step 2
	Index = RightLeg + Tripod	;replaces for/next loop
	
	Rotate(Index) = Rotate(Index) + (Steering/5 - Rotate(Index))/StepFlag
		
	YPos(Index) = -Tibia_Length + Height
	XPos(Index) = XPos(Index) + (XPos2(Index) - (XPos(Index) - (HipV_HipH + Femur_Length)))/StepFlag
	ZPos(Index) = ZPos(Index) + (ZPos2(Index) - ZPos(Index))/StepFlag
;	next

	low LED_D6
return

;--------------------------------------------------------------------
; Up Tripod subroutine
;Tripod 'A' (tripod = 0) or 'B' (Tripod = 1) Up

AorB_Up
	high LED_D6	; flash led to show activity

 	StepFlag = StepFlag - 1

;	for Index = 0 + Tripod to 4 + Tripod Step 2
	Index = RightLeg + Tripod	; replaces for/next 

	Rotate(Index) = Rotate(Index) + (-Steering/5 - Rotate(Index))/StepFlag
		
	YPos(Index) = -Tibia_Length + Height + (LegUpShift * Freeze)
	XPos(Index) = XPos(Index) + (-XPos2(Index) - (XPos(Index) - (HipV_HipH + Femur_Length)))/StepFlag
	ZPos(Index) = ZPos(Index) + (-ZPos2(Index) - ZPos(Index))/StepFlag
;	next

	low LED_D6
		
return	
	
;--------------------------------------------------------------------
; Serial Query (COM) routine

SerialQuery

; LED_D2 will flash slow if PC COM not connected, faster if connected, 
; LED_D4 will flash with when receiving commands (good checksum)

low LED_D4	; low=off

	hserout ["Rd"]	; 'Ready' to recieve data now

	low LED_D2	; "heart beat" ON
	pause 50
	high LED_D2 	; "heart beat" OFF
	pause 50

	hserin NoData,1000,[str DualShock(0)\7]	; get the data
	hserout [str DualShock(0)\7]		; echo for testing
	
	CheckSum = DualShock(1) ^ DualShock(2) ^ DualShock(3) ^ |
		DualShock(4) ^ DualShock(5) ^ DualShock(6)

;	Hserout [str DualShock(0)\7]	; test echo received numbers
;	hserout [CheckSum, cr]		; test

if CheckSum <> DualShock(0) then
	Sound SPKR,[500\2000]	; checksum bad

NoData	; serial port timeouts result in slower LED flashing
	DualShock(1) = $FF	; clear out command buttons
	DualShock(2) = $FF

	for Index = 3 to 6
		DualShock(Index) = $80	; clear out joysticks
	next

else
	high LED_D4	; high=on test good checksum
endif

;	gosub show_xy	; testing, display joystick X and Y positions

	low LED_D4	; LED off

return

;--------------------------------------------------------------------
; PS2 Query routine
; initialize PS2 interface (if needed)

PS2_Query

	high LED_D4	; LED will flash slower if PS2 not connected (?), faster if connected, 
			; and really fast when receiving commands

retry_PS2
	low LED_D2	; "heart beat" ON
	low SEL
	shiftout CMD,CLK,FASTLSBPRE,[$1\8]
	shiftin DAT,CLK,FASTLSBPOST,[DS2Mode\8]	; get the PS2 mode first to see if we need to init
	high SEL
	pause 1

	; send read command to PS2, and get 7 bytes of data
	
	low SEL
	shiftout CMD,CLK,FASTLSBPRE,[$1\8,$42\8]	
	shiftin DAT,CLK,FASTLSBPOST,[DualShock(0)\8, DualShock(1)\8, DualShock(2)\8, DualShock(3)\8, |
		DualShock(4)\8, DualShock(5)\8, DualShock(6)\8]
	high SEL
	pause 1	
	high LED_D2	; "heart beat" OFF

	; if PS2 not in native mode (analog joystick) initialize 
	
	if DS2Mode <> PadMode then
		low SEL
		shiftout CMD,CLK,FASTLSBPRE,[$1\8,$43\8,$0\8,$1\8,$0\8] ;CONFIG_MODE_ENTER
		high SEL
		pause 1

		low SEL
		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$44\8,$00\8,$01\8,$03\8,$00\8,$00\8,$00\8,$00\8] ;SET_MODE_AND_LOCK
		high SEL
		pause 1

		low SEL
		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$4F\8,$00\8,$FF\8,$FF\8,$03\8,$00\8,$00\8,$00\8] ;SET_DS2_NATIVE_MODE
		high SEL
		pause 1

		low SEL
		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$43\8,$00\8,$00\8,$5A\8,$5A\8,$5A\8,$5A\8,$5A\8] ;CONFIG_MODE_EXIT_DS2_NATIVE
		high SEL
		pause 1

		low SEL
		shiftout CMD,CLK,FASTLSBPRE,[$01\8,$43\8,$00\8,$00\8,$00\8,$00\8,$00\8,$00\8,$00\8] ;CONFIG_MODE_EXIT
		high SEL
		pause 288

		goto retry_PS2
	endif

	low LED_D4

return
;--------------------------------------------------------------------
; show right joystick Left and right values (testing)
show_xy

; XSpeed Right Stick Horizontal
; YSpeed Right Stick Vertical


	LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1,[HOME]	; home
;	pause 50

	LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1,["X ", sdec3 XSpeed,"    "]
;	pause 50
	
	LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1,[SCRRAM+$40]	; second line
;	pause 50
	
	LCDWRITE LCD_RS\LCD_E\LCD_RW, portd.nib1,["Y ", sdec3 YSpeed,"    "]
;	pause 50

return
;--------------------------------------------------------------------
